@@ -57,7 +57,6 @@
 #include "mipict.h"
 #include "fbpict.h"
 
-extern int RenderErrBase;
 extern int (*ProcRenderVector[RenderNumberRequests]) (ClientPtr);
 
 static int (*dmxSaveRenderVector[RenderNumberRequests]) (ClientPtr);
@@ -141,10 +140,6 @@ dmxPictureInit(ScreenPtr pScreen, PictFormatPtr formats, int nformats)
     DMXScreenInfo *dmxScreen = &dmxScreens[pScreen->myNum];
     PictureScreenPtr ps;
 
-    /* The shadow framebuffer only relies on FB to be initialized */
-    if (dmxShadowFB)
-        return fbPictureInit(pScreen, formats, nformats);
-
     if (!miPictureInit(pScreen, formats, nformats))
         return FALSE;
 
@@ -289,7 +284,7 @@ dmxProcRenderCreateGlyphSet(ClientPtr client)
         /* Store glyphsets from backends in glyphSet->devPrivate ????? */
         /* Make sure we handle all errors here!! */
 
-        dixLookupResourceByType((pointer *) &glyphSet,
+        dixLookupResourceByType((void **) &glyphSet,
                                 stuff->gsid, GlyphSetType,
                                 client, DixDestroyAccess);
 
@@ -336,7 +331,7 @@ dmxProcRenderFreeGlyphSet(ClientPtr client)
     REQUEST(xRenderFreeGlyphSetReq);
 
     REQUEST_SIZE_MATCH(xRenderFreeGlyphSetReq);
-    dixLookupResourceByType((pointer *) &glyphSet,
+    dixLookupResourceByType((void **) &glyphSet,
                             stuff->glyphset, GlyphSetType,
                             client, DixDestroyAccess);
 
@@ -382,7 +377,7 @@ dmxProcRenderAddGlyphs(ClientPtr client)
         CARD8 *bits;
         int nbytes;
 
-        dixLookupResourceByType((pointer *) &glyphSet,
+        dixLookupResourceByType((void **) &glyphSet,
                                 stuff->glyphset, GlyphSetType,
                                 client, DixReadAccess);
         glyphPriv = DMX_GET_GLYPH_PRIV(glyphSet);
@@ -395,7 +390,7 @@ dmxProcRenderAddGlyphs(ClientPtr client)
                   sizeof(xRenderAddGlyphsReq) -
                   (sizeof(CARD32) + sizeof(xGlyphInfo)) * nglyphs);
 
-        gidsCopy = malloc(sizeof(*gidsCopy) * nglyphs);
+        gidsCopy = xallocarray(nglyphs, sizeof(*gidsCopy));
         for (i = 0; i < nglyphs; i++)
             gidsCopy[i] = gids[i];
 
@@ -427,7 +422,7 @@ dmxProcRenderFreeGlyphs(ClientPtr client)
     REQUEST(xRenderFreeGlyphsReq);
 
     REQUEST_AT_LEAST_SIZE(xRenderFreeGlyphsReq);
-    dixLookupResourceByType((pointer *) &glyphSet,
+    dixLookupResourceByType((void **) &glyphSet,
                             stuff->glyphset, GlyphSetType,
                             client, DixWriteAccess);
 
@@ -439,7 +434,7 @@ dmxProcRenderFreeGlyphs(ClientPtr client)
 
         nglyphs = ((client->req_len << 2) - sizeof(xRenderFreeGlyphsReq)) >> 2;
         if (nglyphs) {
-            gids = malloc(sizeof(*gids) * nglyphs);
+            gids = xallocarray(nglyphs, sizeof(*gids));
             for (i = 0; i < nglyphs; i++)
                 gids[i] = ((CARD32 *) (stuff + 1))[i];
 
@@ -502,14 +497,14 @@ dmxProcRenderCompositeGlyphs(ClientPtr client)
         GlyphSetPtr glyphSet;
         dmxGlyphPrivPtr glyphPriv;
 
-        dixLookupResourceByType((pointer *) &pSrc,
+        dixLookupResourceByType((void **) &pSrc,
                                 stuff->src, PictureType, client, DixReadAccess);
 
         pSrcPriv = DMX_GET_PICT_PRIV(pSrc);
         if (!pSrcPriv->pict)
             return ret;
 
-        dixLookupResourceByType((pointer *) &pDst,
+        dixLookupResourceByType((void **) &pDst,
                                 stuff->dst, PictureType,
                                 client, DixWriteAccess);
 
@@ -528,7 +523,7 @@ dmxProcRenderCompositeGlyphs(ClientPtr client)
             return ret;
 
         if (stuff->maskFormat)
-            dixLookupResourceByType((pointer *) &pFmt,
+            dixLookupResourceByType((void **) &pFmt,
                                     stuff->maskFormat, PictFormatType,
                                     client, DixReadAccess);
         else
@@ -574,11 +569,11 @@ dmxProcRenderCompositeGlyphs(ClientPtr client)
         /* The following only works for Render version > 0.2 */
 
         /* All of the XGlyphElt* structure sizes are identical */
-        elts = malloc(nelt * sizeof(XGlyphElt8));
+        elts = xallocarray(nelt, sizeof(XGlyphElt8));
         if (!elts)
             return BadAlloc;
 
-        glyphs = malloc(nglyph * size);
+        glyphs = xallocarray(nglyph, size);
         if (!glyphs) {
             free(elts);
             return BadAlloc;
@@ -589,7 +584,7 @@ dmxProcRenderCompositeGlyphs(ClientPtr client)
         curGlyph = glyphs;
         curElt = elts;
 
-        dixLookupResourceByType((pointer *) &glyphSet,
+        dixLookupResourceByType((void **) &glyphSet,
                                 stuff->glyphset, GlyphSetType,
                                 client, DixReadAccess);
         glyphPriv = DMX_GET_GLYPH_PRIV(glyphSet);
@@ -599,7 +594,7 @@ dmxProcRenderCompositeGlyphs(ClientPtr client)
             buffer += sizeof(xGlyphElt);
 
             if (elt->len == 0xff) {
-                dixLookupResourceByType((pointer *) &glyphSet,
+                dixLookupResourceByType((void **) &glyphSet,
                                         *((CARD32 *) buffer),
                                         GlyphSetType, client, DixReadAccess);
                 glyphPriv = DMX_GET_GLYPH_PRIV(glyphSet);
@@ -898,7 +893,7 @@ dmxDestroyPicture(PicturePtr pPicture)
 
 /** Change the picture's list of clip rectangles. */
 int
-dmxChangePictureClip(PicturePtr pPicture, int clipType, pointer value, int n)
+dmxChangePictureClip(PicturePtr pPicture, int clipType, void *value, int n)
 {
     ScreenPtr pScreen = pPicture->pDrawable->pScreen;
     DMXScreenInfo *dmxScreen = &dmxScreens[pScreen->myNum];
@@ -930,7 +925,7 @@ dmxChangePictureClip(PicturePtr pPicture, int clipType, pointer value, int n)
             int nRects;
 
             nRects = nBox;
-            pRects = pRect = malloc(nRects * sizeof(*pRect));
+            pRects = pRect = xallocarray(nRects, sizeof(*pRect));
 
             while (nBox--) {
                 pRect->x = pBox->x1;
