@@ -88,12 +88,6 @@ ResetClientState(int clientIndex)
     memset(cl, 0, sizeof(__GLXclientState));
     cl->be_displays = keep_be_displays;
 
-    /*
-     ** By default, assume that the client supports
-     ** GLX major version 1 minor version 0 protocol.
-     */
-    cl->GLClientmajorVersion = 1;
-    cl->GLClientminorVersion = 0;
     free(cl->GLClientextensions);
 
     memset(cl->be_displays, 0, screenInfo.numScreens * sizeof(Display *));
@@ -188,7 +182,7 @@ __glXFreeGLXWindow(__glXWindow * pGlxWindow)
         WindowPtr pWindow = (WindowPtr) pGlxWindow->pDraw;
         WindowPtr ret;
 
-        dixLookupResourceByType((pointer) &ret,
+        dixLookupResourceByType((void *) &ret,
                                 pWindow->drawable.id, RT_WINDOW,
                                 NullClient, DixUnknownAccess);
         if (ret == pWindow) {
@@ -353,7 +347,7 @@ GlxExtensionInit(void)
     /*
      ** Initialize table of client state.  There is never a client 0.
      */
-    for (i = 1; i <= MAXCLIENTS; i++) {
+    for (i = 1; i <= LimitClients; i++) {
         __glXClients[i] = 0;
     }
 
@@ -420,7 +414,7 @@ __glXDispatch(ClientPtr client)
          */
         XID xid = FakeClientID(client->index);
 
-        if (!AddResource(xid, __glXClientRes, (pointer) (long) client->index)) {
+        if (!AddResource(xid, __glXClientRes, (void *) (long) client->index)) {
             return BadAlloc;
         }
         ResetClientState(client->index);
@@ -474,7 +468,7 @@ __glXSwapDispatch(ClientPtr client)
          */
         XID xid = FakeClientID(client->index);
 
-        if (!AddResource(xid, __glXClientRes, (pointer) (long) client->index)) {
+        if (!AddResource(xid, __glXClientRes, (void *) (long) client->index)) {
             return BadAlloc;
         }
         ResetClientState(client->index);
