@@ -26,13 +26,13 @@ Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts.
 
                         All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of Digital not be
 used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.  
+software without specific, written prior permission.
 
 DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
@@ -65,7 +65,7 @@ extern _X_EXPORT int CountBits(const uint8_t * mask, int len);
 #define SameClient(obj,client) \
 	(CLIENT_BITS((obj)->resource) == (client)->clientAsMask)
 
-#define EMASKSIZE	MAXDEVICES + 2
+#define EMASKSIZE	(MAXDEVICES + 2)
 
 /* This is the last XI2 event supported by the server. If you add
  * events to the protocol, the server will not support these events until
@@ -159,7 +159,7 @@ typedef struct _OtherInputMasks {
  */
 
 #define MasksPerDetailMask 8    /* 256 keycodes and 256 possible
-                                   modifier combinations, but only      
+                                   modifier combinations, but only
                                    3 buttons. */
 
 typedef struct _DetailRec {     /* Grab details may be bit masks */
@@ -174,9 +174,9 @@ union _GrabMask {
 };
 
 /**
- * Central struct for device grabs. 
+ * Central struct for device grabs.
  * The same struct is used for both core grabs and device grabs, with
- * different fields being set. 
+ * different fields being set.
  * If the grab is a core grab (GrabPointer/GrabKeyboard), then the eventMask
  * is a combination of standard event masks (i.e. PointerMotionMask |
  * ButtonPressMask).
@@ -447,7 +447,7 @@ typedef struct _XIPropertyValue {
     Atom type;                  /* ignored by server */
     short format;               /* format of data for swapping - 8,16,32 */
     long size;                  /* size of data in (format/8) bytes */
-    pointer data;               /* private to client */
+    void *data;                 /* private to client */
 } XIPropertyValueRec;
 
 typedef struct _XIProperty {
