@@ -132,7 +132,6 @@ typedef struct _Picture {
     unsigned int polyEdge:1;
     unsigned int polyMode:1;
     unsigned int freeCompClip:1;
-    unsigned int clientClipType:2;
     unsigned int componentAlpha:1;
     unsigned int repeatType:2;
     unsigned int filter:3;
@@ -145,7 +144,7 @@ typedef struct _Picture {
     DDXPointRec alphaOrigin;
 
     DDXPointRec clipOrigin;
-    pointer clientClip;
+    RegionPtr clientClip;
 
     unsigned long serialNumber;
 
@@ -189,7 +188,7 @@ typedef struct {
 typedef int (*CreatePictureProcPtr) (PicturePtr pPicture);
 typedef void (*DestroyPictureProcPtr) (PicturePtr pPicture);
 typedef int (*ChangePictureClipProcPtr) (PicturePtr pPicture,
-                                         int clipType, pointer value, int n);
+                                         int clipType, void *value, int n);
 typedef void (*DestroyPictureClipProcPtr) (PicturePtr pPicture);
 
 typedef int (*ChangePictureTransformProcPtr) (PicturePtr pPicture,
@@ -345,16 +344,14 @@ typedef struct _PictureScreen {
 } PictureScreenRec, *PictureScreenPtr;
 
 extern _X_EXPORT DevPrivateKeyRec PictureScreenPrivateKeyRec;
-
 #define PictureScreenPrivateKey (&PictureScreenPrivateKeyRec)
 
 extern _X_EXPORT DevPrivateKeyRec PictureWindowPrivateKeyRec;
-
 #define	PictureWindowPrivateKey (&PictureWindowPrivateKeyRec)
 
-extern _X_EXPORT RESTYPE PictureType;
-extern _X_EXPORT RESTYPE PictFormatType;
-extern _X_EXPORT RESTYPE GlyphSetType;
+extern RESTYPE PictureType;
+extern RESTYPE PictFormatType;
+extern RESTYPE GlyphSetType;
 
 #define GetPictureScreen(s) ((PictureScreenPtr)dixLookupPrivate(&(s)->devPrivates, PictureScreenPrivateKey))
 #define GetPictureScreenIfSet(s) (dixPrivateKeyRegistered(PictureScreenPrivateKey) ? GetPictureScreen(s) : NULL)
@@ -363,10 +360,10 @@ extern _X_EXPORT RESTYPE GlyphSetType;
 #define SetPictureWindow(w,p) dixSetPrivate(&(w)->devPrivates, PictureWindowPrivateKey, p)
 
 #define VERIFY_PICTURE(pPicture, pid, client, mode) {\
-    int rc = dixLookupResourceByType((pointer)&(pPicture), pid,\
-	                             PictureType, client, mode);\
-    if (rc != Success)\
-	return rc;\
+    int tmprc = dixLookupResourceByType((void *)&(pPicture), pid,\
+	                                PictureType, client, mode);\
+    if (tmprc != Success)\
+	return tmprc;\
 }
 
 #define VERIFY_ALPHA(pPicture, pid, client, mode) {\
@@ -377,17 +374,8 @@ extern _X_EXPORT RESTYPE GlyphSetType;
     } \
 } \
 
-extern _X_EXPORT Bool
- PictureDestroyWindow(WindowPtr pWindow);
-
-extern _X_EXPORT Bool
- PictureCloseScreen(int Index, ScreenPtr pScreen);
-
-extern _X_EXPORT void
- PictureStoreColors(ColormapPtr pColormap, int ndef, xColorItem * pdef);
-
-extern _X_EXPORT Bool
- PictureInitIndexedFormat(ScreenPtr pScreen, PictFormatPtr format);
+extern _X_EXPORT PictFormatPtr
+ PictureWindowFormat(WindowPtr pWindow);
 
 extern _X_EXPORT Bool
  PictureSetSubpixelOrder(ScreenPtr pScreen, int subpixel);
@@ -395,9 +383,6 @@ extern _X_EXPORT Bool
 extern _X_EXPORT int
  PictureGetSubpixelOrder(ScreenPtr pScreen);
 
-extern _X_EXPORT PictFormatPtr
-PictureCreateDefaultFormats(ScreenPtr pScreen, int *nformatp);
-
 extern _X_EXPORT PictFormatPtr
 PictureMatchVisual(ScreenPtr pScreen, int depth, VisualPtr pVisual);
 
@@ -413,14 +398,12 @@ extern _X_EXPORT int
 extern _X_EXPORT char *PictureGetFilterName(int id);
 
 extern _X_EXPORT int
-
 PictureAddFilter(ScreenPtr pScreen,
                  const char *filter,
                  PictFilterValidateParamsProcPtr ValidateParams,
                  int width, int height);
 
 extern _X_EXPORT Bool
-
 PictureSetFilterAlias(ScreenPtr pScreen, const char *filter, const char *alias);
 
 extern _X_EXPORT Bool
@@ -433,21 +416,16 @@ extern _X_EXPORT PictFilterPtr
 PictureFindFilter(ScreenPtr pScreen, char *name, int len);
 
 extern _X_EXPORT int
-
 SetPicturePictFilter(PicturePtr pPicture, PictFilterPtr pFilter,
                      xFixed * params, int nparams);
 
 extern _X_EXPORT int
-
 SetPictureFilter(PicturePtr pPicture, char *name, int len,
                  xFixed * params, int nparams);
 
 extern _X_EXPORT Bool
  PictureFinishInit(void);
 
-extern _X_EXPORT void
- SetPictureToDefaults(PicturePtr pPicture);
-
 extern _X_EXPORT PicturePtr
 CreatePicture(Picture pid,
               DrawablePtr pDrawable,
@@ -455,7 +433,6 @@ CreatePicture(Picture pid,
               Mask mask, XID *list, ClientPtr client, int *error);
 
 extern _X_EXPORT int
-
 ChangePicture(PicturePtr pPicture,
               Mask vmask, XID *vlist, DevUnion *ulist, ClientPtr client);
 
@@ -465,27 +442,19 @@ SetPictureClipRects(PicturePtr pPicture,
                     int xOrigin, int yOrigin, int nRect, xRectangle *rects);
 
 extern _X_EXPORT int
-
 SetPictureClipRegion(PicturePtr pPicture,
                      int xOrigin, int yOrigin, RegionPtr pRegion);
 
 extern _X_EXPORT int
  SetPictureTransform(PicturePtr pPicture, PictTransform * transform);
 
-extern _X_EXPORT void
- CopyPicture(PicturePtr pSrc, Mask mask, PicturePtr pDst);
-
 extern _X_EXPORT void
  ValidatePicture(PicturePtr pPicture);
 
 extern _X_EXPORT int
- FreePicture(pointer pPicture, XID pid);
-
-extern _X_EXPORT int
- FreePictFormat(pointer pPictFormat, XID pid);
+ FreePicture(void *pPicture, XID pid);
 
 extern _X_EXPORT void
-
 CompositePicture(CARD8 op,
                  PicturePtr pSrc,
                  PicturePtr pMask,
@@ -497,7 +466,6 @@ CompositePicture(CARD8 op,
                  INT16 xDst, INT16 yDst, CARD16 width, CARD16 height);
 
 extern _X_EXPORT void
-
 CompositeGlyphs(CARD8 op,
                 PicturePtr pSrc,
                 PicturePtr pDst,
@@ -506,13 +474,11 @@ CompositeGlyphs(CARD8 op,
                 INT16 ySrc, int nlist, GlyphListPtr lists, GlyphPtr * glyphs);
 
 extern _X_EXPORT void
-
 CompositeRects(CARD8 op,
                PicturePtr pDst,
                xRenderColor * color, int nRect, xRectangle *rects);
 
 extern _X_EXPORT void
-
 CompositeTrapezoids(CARD8 op,
                     PicturePtr pSrc,
                     PicturePtr pDst,
@@ -520,7 +486,6 @@ CompositeTrapezoids(CARD8 op,
                     INT16 xSrc, INT16 ySrc, int ntrap, xTrapezoid * traps);
 
 extern _X_EXPORT void
-
 CompositeTriangles(CARD8 op,
                    PicturePtr pSrc,
                    PicturePtr pDst,
@@ -529,7 +494,6 @@ CompositeTriangles(CARD8 op,
                    INT16 ySrc, int ntriangles, xTriangle * triangles);
 
 extern _X_EXPORT void
-
 CompositeTriStrip(CARD8 op,
                   PicturePtr pSrc,
                   PicturePtr pDst,
@@ -537,25 +501,20 @@ CompositeTriStrip(CARD8 op,
                   INT16 xSrc, INT16 ySrc, int npoints, xPointFixed * points);
 
 extern _X_EXPORT void
-
 CompositeTriFan(CARD8 op,
                 PicturePtr pSrc,
                 PicturePtr pDst,
                 PictFormatPtr maskFormat,
                 INT16 xSrc, INT16 ySrc, int npoints, xPointFixed * points);
 
-extern _X_EXPORT void RenderExtensionInit(void);
-
 Bool
  AnimCurInit(ScreenPtr pScreen);
 
 int
-
 AnimCursorCreate(CursorPtr *cursors, CARD32 *deltas, int ncursor,
                  CursorPtr *ppCursor, ClientPtr client, XID cid);
 
 extern _X_EXPORT void
-
 AddTraps(PicturePtr pPicture,
          INT16 xOff, INT16 yOff, int ntraps, xTrap * traps);
 
@@ -586,8 +545,8 @@ CreateConicalGradientPicture(Picture pid,
                              xFixed * stops, xRenderColor * colors, int *error);
 
 #ifdef PANORAMIX
-extern _X_EXPORT void PanoramiXRenderInit(void);
-extern _X_EXPORT void PanoramiXRenderReset(void);
+extern void PanoramiXRenderInit(void);
+extern void PanoramiXRenderReset(void);
 #endif
 
 /*
@@ -595,12 +554,10 @@ extern _X_EXPORT void PanoramiXRenderReset(void);
  */
 
 extern _X_EXPORT void
-
 PictTransform_from_xRenderTransform(PictTransformPtr pict,
                                     xRenderTransform * render);
 
 extern _X_EXPORT void
-
 xRenderTransform_from_PictTransform(xRenderTransform * render,
                                     PictTransformPtr pict);
 
